//
// 
// CPPBENCH.CPP
// 
// Copyright (c) 1998,99 On Time
// http://www.on-time.com
// 
// Benchmark for C++ Exception Handling
// 
// 

#include <windows.h>
#include <stdlib.h>
#include <stdio.h>

#define DIVIDE_BY_ZERO -3 // an error code

// some global variables to count
// iterations, exceptions, and cleanups

int Calls, Exceptions, Finallys;

///////////////////////////////////////
int SomeFunction(int a, int b)
// functiom which can raise an exception
{
   if (b == 0)
      throw DIVIDE_BY_ZERO;
   return a / b;
}

///////////////////////////////////////
int TestFunction(int a, int b)
// test function containing try block
{
   int Result;

   class Finally {
   public:
      Finally()  { Calls++; }
      ~Finally() { Finallys++; }
   } FinallyHandler;

   try
   {
      Result = SomeFunction(a, b);
   }
   catch (int& ErrorCode)
   {
      switch (ErrorCode)
      {
         case DIVIDE_BY_ZERO:
            Exceptions++;
            break;
         default:
            printf("unknown exception!\n");
            throw;
       }
   }
   catch (...)
   {
      printf("non integer exception!\n");
      throw;
   }
   return Result;
}

///////////////////////////////////////
void Bench(int a, int b, int Iterations)
// benchmark function to call TestFunction in a loop
// and print timing and statistics 
{
   DWORD T0, T1;
   int i;

   Calls = Exceptions = Finallys = 0;
   T0 = GetTickCount();
   for (i=0; i<Iterations; i++)
      TestFunction(a, b);
   T1 = GetTickCount();
   printf("%10i  %10i  %10i    %10u\n", Calls, Exceptions, Finallys, T1-T0);
}

///////////////////////////////////////
int main(void)
{
   printf("Interation  Exceptions    Finallys  Milliseconds\n"
          "------------------------------------------------\n");
   Bench(1, 1, 1000000);   // no exceptions
   Bench(1, 0, 1000000);   // raise one exception per loop
   return 0;
}
