// CopyCmd.cpp: implementation of the CCopyCmd class.
//
//////////////////////////////////////////////////////////////////////

/*
 *	Martin Hallerdal, marty@home.se
 *	2000-05-11
 *
 *	Please feel free to experiment with the code. Just don't use it for commercial purposes.
 */


#include "stdafx.h"
#include "DavFilter.h"
#include "CopyCmd.h"
#include "XMLBody.h"
#include "HttpUtil.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CCopyCmd::CCopyCmd(CDavCmdDispatcher* disp, CHttpFilterContext* pCtx) :  CDavCmd(disp, pCtx)
{

}

CCopyCmd::~CCopyCmd()
{
	
}

/*
	Performs a COPY command
	NOTE: The request may have a body but we ignore it because it doesn't matter to us.
*/


bool CCopyCmd::Execute() {
	

	const char* szOW = pDisp->pHdrs->GetHeader("Overwrite");
	
	bOverwrite = szOW ? ((*szOW == 'T') ? true : false) : false; 
	
	//Find the destination header
	
	const char* szDest = pDisp->pHdrs->GetHeader("Destination");

	URLDecode((char*)szDest);
	
	char szPhysDest[MAX_PATH];

	do {
		if(!MapPath(szDest, szPhysDest, pCtx)) {
			resp.SetStatus(502);
			break;
		} 
		
		//Find the destination URI
		
		const char* szURI = strstr(szDest, "://");
		szURI += 3;
		szURI = strchr(szURI, '/');
		
		char szStackURI[512];
		strcpy(szStackURI, szURI);
		
		//Check if destination and source are the same
	
		char* szDestURI = strstr(szDest, "://");
		
		if(szDestURI) {
			szDestURI = strchr(szDestURI+3, '/');
			
			if(szDestURI) {
				if(!_stricmp(szDestURI, pDisp->szURI)) {
					resp.SetStatus(403);	
					break;
				}
			}
			
		}
		
		
		//Does the parent of the destination resource exist?
		
		if(!AncestorExist(szPhysDest)) {
			resp.SetStatus(409);
			break;			
		} 		
		
		
		//Get the depth
		
		const char* szDepth = pDisp->pHdrs->GetHeader("Depth");
		iDepth = 2;
		
		if(szDepth) {
			iDepth = atoi(szDepth);
		}
		
		
		CopyResource(pDisp->szPhys, szPhysDest, szStackURI, true);
		
		
	} while(false);
	

	//Add a response body if necessary

	if(bDidInit) {
		doc->documentElement = elemRoot;
		
		CXMLBody* pResp = new CXMLBody;
		pResp->SetXMLDocument(doc);
		
		_bstr_t ba = doc->xml;
		char* p = (char*)ba;
		
		resp.pBody = pResp;
	}
				
				
				
	resp.SendResponse(pCtx);
	return true;
}

/*
	Copies a file
*/

void CCopyCmd::CopyResource(char* szSrc, char* szDst, char* szDstURI, bool bFirst) {
	
	
	BOOL bRet;
	DWORD dwSrcLen = strlen(szSrc);
	DWORD dwDstLen = strlen(szDst);
	DWORD dwDstURILen = strlen(szDstURI);
	bool bExist = true;

	if(szSrc[dwSrcLen-1] == '\\') {
		szSrc[dwSrcLen-1] = '\0';
		dwSrcLen--;
	}

	//Get file attributes

	DWORD dwAtts = ::GetFileAttributes(szSrc);

	if(dwAtts == 0xFFFFFFFF) {
		SetError(bFirst, szDstURI);
		return;
	}
	
	bool bIsDir = (dwAtts & FILE_ATTRIBUTE_DIRECTORY) ? true : false;

	//Does the destination exist?
	
	DWORD dwAttsDst = ::GetFileAttributes(szDst);
	if(dwAttsDst == 0xFFFFFFFF) {
		if(::GetLastError() == ERROR_FILE_NOT_FOUND || ::GetLastError() == ERROR_PATH_NOT_FOUND) {
			bExist = false;
		}
	}
	
	if(bExist && !bOverwrite) {
		SetError(bFirst, szDstURI, 412);
		return;
	}
	
	//Try to copy the file or create the directory
	
	if(bIsDir) {
		
		bRet = TRUE;

		if(!bExist) {
			bRet = ::CreateDirectoryEx(szSrc, szDst, NULL);
		}
	
	} else {
		bRet = ::CopyFile(szSrc, szDst, bOverwrite);
	}

		
	if(bRet == FALSE) {
		SetError(bFirst, szDstURI);
		return;
	} 
	
	//Set appropriate status
	
	if(bFirst) {
		if(bExist) {
			resp.SetStatus(204);
		} else {
			resp.SetStatus(201);
		}
	}
	
	//If directory copy members
	
	if(bIsDir && iDepth != 0) {
		
		
		szSrc[dwSrcLen] = '\\';
		szSrc[dwSrcLen+1] = '\0';
		dwSrcLen++;

		strcat(szSrc, "*.*");
		
		szDstURI[dwDstURILen] = '/';
		szDstURI[dwDstURILen+1] = '\0';
		dwDstURILen++;
		
		szDst[dwDstLen] = '\\';
		szDst[dwDstLen+1] = '\0';
		dwDstLen++;
		
		//Find files in directory
		
		WIN32_FIND_DATA fdata;
		BOOL bFind;			
		
	
		HANDLE hFind = ::FindFirstFile(szSrc, &fdata);
		
		szSrc[dwSrcLen] = '\0';
		
		if(hFind != INVALID_HANDLE_VALUE) {
			
			//Loop through the files
			
			while(true) {
				
				if(strcmp(fdata.cFileName, ".") && strcmp(fdata.cFileName, "..")) {
					
					//Restore strings
					
					szDst[dwDstLen] = '\0';
					szSrc[dwSrcLen] = '\0';
					szDstURI[dwDstURILen] = '\0';
					
					strcat(szDstURI, fdata.cFileName);
					strcat(szDst, fdata.cFileName);
					strcat(szSrc, fdata.cFileName);
					
					
					CopyResource(szSrc, szDst, szDstURI, false);
					
				}
				
				
				//Get the next file
				
				bFind = ::FindNextFile(hFind, &fdata);
				
				if(bFind == FALSE) {
					if(::GetLastError() == ERROR_NO_MORE_FILES) {
						break;
					}
				}
			}

			::FindClose(hFind);	

		}	//Invalid HANDLE value
	
		
		
		
	} //Is directory
	
}

/*
	Returns an appropriate HTTP status code
*/

int CCopyCmd::GetHttpStatus(DWORD dwErrcode) {
	
	switch(dwErrcode) {

	case ERROR_ALREADY_EXISTS:
		return 412;
	
	default:
		return CDavCmd::GetHttpStatus(dwErrcode);
	}
}