// DavCmd.cpp: implementation of the CDavCmd class.
//
//////////////////////////////////////////////////////////////////////

/*
 *	Martin Hallerdal, marty@home.se
 *	2000-05-11
 *
 *	Please feel free to experiment with the code. Just don't use it for commercial purposes.
 */


#include "stdafx.h"
#include "HttpUtil.h"
#include "DavCmd.h"

#include <atlbase.h>
#include <iads.h>
#include <adshlp.h>

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

using namespace MSXML;

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CDavCmd::CDavCmd(CDavCmdDispatcher* disp, CHttpFilterContext* pCtx) : bDidInit(false) {
	
	pDisp = disp;
	this->pCtx = pCtx;
	
}

CDavCmd::~CDavCmd()
{

}

/* Parses the request body and returns an XML-element */

IXMLDOMElementPtr CDavCmd::GetRootXMLElement() {

	IXMLDOMElementPtr elemRoot;
	IXMLDOMDocumentPtr doc;
	HRESULT hr;
	VARIANT_BOOL bRet;

	
	//Does the body contain an XML document?

	const char* szXMLStart = strstr(pDisp->szBody, "<?xml");
	
	if(szXMLStart == NULL) {
		return NULL;
	}
	
	szXMLStart = strstr(szXMLStart, "?>");
	
	if(szXMLStart == NULL) {
		return NULL;
	}

	szXMLStart += 2;

	//Create object

	hr = doc.CreateInstance("MSXML.DOMDocument");

	if(SUCCEEDED(hr)) {
		
		bRet = doc->loadXML(szXMLStart);
		
		if(bRet == -1) {
			elemRoot = doc->documentElement;
		}
	}

	return elemRoot;
}



/*
	Checks if the parent directory exists
*/
bool CDavCmd::AncestorExist(char* szPath) {

	if(szPath == NULL) {
		szPath = pDisp->szPhys;
	}

	char cOld;
	DWORD dwLen = strlen(szPath);
	char* szPtr = (szPath + (dwLen-1));

	if(*szPtr == '\\') {
		szPtr--;
	}

	//Find parent directory

	while(*szPtr != '\\' && *szPtr != '\0') {
		*szPtr--;
	}


	//If we found a parent directory

	if(*szPtr) {
		cOld = *szPtr;
		*szPtr = '\0';

		DWORD dwAtt = ::GetFileAttributes(szPath);
		
		if(dwAtt == 0xFFFFFFFF) {

			if(::GetLastError() == ERROR_FILE_NOT_FOUND || ::GetLastError() == ERROR_PATH_NOT_FOUND) {
				//Dir doesn't exist.

				return false;
			}
		}

		//Restore
		
		*szPtr = cOld;
	}

	
	return true;
}

/*
	Sets error in response
*/

void CDavCmd::SetError(bool bFirst, char* szURI, DWORD dwStatus) {

	if(dwStatus == 0) {
		DWORD dwErr = ::GetLastError();
		dwStatus = GetHttpStatus(dwErr);
	}

	if(bFirst) {
		
		resp.SetStatus(dwStatus);
	} else {
		resp.SetStatus(207);

		
		//Create a document if it isn't initied
		
		if(!bDidInit) {
			doc.CreateInstance("MSXML.DOMDocument");
			elemRoot = doc->createNode(1L, "D:multistatus", "DAV");

			bDidInit = true;
		}



		IXMLDOMNodePtr elemResponse = doc->createElement("D:response");

		IXMLDOMNodePtr elemHref = doc->createElement("D:href");
		IXMLDOMNodePtr elemStatus = doc->createElement("D:status");
	

		IXMLDOMTextPtr elemText = doc->createTextNode(::GetStatusLineFromCode(dwStatus));
		elemStatus->appendChild(elemText);

		elemText = doc->createTextNode(szURI);
		elemHref->appendChild(elemText);

		elemResponse->appendChild(elemHref);
		elemResponse->appendChild(elemStatus);


		elemRoot->appendChild(elemResponse);
	}
}

/*
	Returns an appropriate HTTP status code
*/

int CDavCmd::GetHttpStatus(DWORD dwErrcode) {
	
	
	
	switch(dwErrcode) {
		
	case ERROR_FILE_NOT_FOUND:
		return 404;;
		break;
		
	case ERROR_PATH_NOT_FOUND:
		return 404;
		break;
		
	case ERROR_ACCESS_DENIED:
		return 401;
		break;
		
	default:
		return 200;
		break;
	}
	
}