// DavCommand.cpp: implementation of the CDavCmdDispatcher class.
//
//////////////////////////////////////////////////////////////////////

/*
 *	Martin Hallerdal, marty@home.se
 *	2000-05-11
 *
 *	Please feel free to experiment with the code. Just don't use it for commercial purposes.
 */


#include "stdafx.h"
#include "DavFilter.h"

#include "DavCmdDispatcher.h"
#include "DavCmd.h"
#include "PropFindCmd.h"
#include "MkColCmd.h"
#include "DeleteCmd.h"
#include "CopyCmd.h"
#include "MoveCmd.h"
#include "HttpUtil.h"

//#include "mtx.h"
#include <comdef.h>

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CDavCmdDispatcher::CDavCmdDispatcher() : pHdrs(NULL), szMethod(NULL), cmd(NOT_INIT), bLoggedOn(false)
{
}

CDavCmdDispatcher::~CDavCmdDispatcher()
{
	SAFE_FREE_PTR(pHdrs)

	//Don't delete szBody, szMethod they're not ours!
}

/*
	Sets the headers for this command
*/

void CDavCmdDispatcher::SetHeaders(CHeaders *pHdrs) {
	this->pHdrs = pHdrs;
}

/*
	Set the requested method
*/

void CDavCmdDispatcher::SetMethod(char *szMethod) { 
	this->szMethod = szMethod;

	if(_stricmp(szMethod,"mkcol") == 0) cmd = MKCOL;
	if(_stricmp(szMethod,"copy") == 0) cmd = COPY;
	if(_stricmp(szMethod,"move") == 0) cmd = MOVE;
	if(_stricmp(szMethod,"delete") == 0) cmd = DEL;
	if(_stricmp(szMethod,"propfind") == 0) cmd = PROPFIND;
}

/*
	Set the URI for this command
*/

void CDavCmdDispatcher::SetURI(char *szNewURI) {

	memset(szURI, 0, 512);
	strcpy(szURI, szNewURI);

	//We will decode this URI

	URLDecode(szURI);
}


/*
	Set the physical location for the URI 
*/

void CDavCmdDispatcher::SetPhysicalLocation(const char *szPhys, int iLen) {
	
	memset(this->szPhys, 0, MAX_PATH);
	strcpy(this->szPhys, szPhys);
	
}

/*
	Checks if the command is a dav command
*/

bool CDavCmdDispatcher::IsDavCommand() {
	return (cmd != NOT_DAV && cmd != NOT_INIT);
}

/*
	Sets the command body
*/

void CDavCmdDispatcher::SetCommandBody(char *szData, DWORD dwLength)
{
	szBody = szData;
	dwBodyLen = dwLength;
}

/*
	Start the show
*/

bool CDavCmdDispatcher::Execute(CHttpFilterContext* pCtx)
{
	bool bRet = false;

	if(!IsDavCommand()) {
		return bRet;
	}

	this->pCtx = pCtx;

	//If the user has requested authentication, perform it

	if(!SetSecurity()) {
		//Set 401 for bad authentication

		CResponse resp;
		resp.SetStatus(401);
		resp.SendResponse(pCtx);

		return bRet;
	}

	//Get a suitable command object for the requested method

	CDavCmd* pCmd = GetCommandInstance();

	if(pCmd) {
		bRet = pCmd->Execute();
		delete pCmd;
	}

	//Revert to self
	
	if(bLoggedOn) {
		RevertToSelf();
		CloseHandle(hUser);
		bLoggedOn = false;
	}

	return bRet;
}

/*
	Impersonate a user if requested
	
	Returns false if impersonation failed
*/

bool CDavCmdDispatcher::SetSecurity() {

	//If we are not secure that we are not doing anything

	if(!pCtx->m_pFC->fIsSecurePort) {
//		return true;
	}

	bLoggedOn = false;
	BOOL b = FALSE;

	const char* szUser = pHdrs->GetBasicUser();
	const char* szPass = pHdrs->GetBasicPassword();

	if(!szUser || !szPass) {
		return true;
	}

	//Look up account

	SID_NAME_USE snu;	
	BYTE sidbuffer[100];
	SID* s = (SID*) sidbuffer;
	DWORD dwSIDLen = 100;
	DWORD dwDomainLen = 50;
	char szDomain[50];


	b = ::LookupAccountName(NULL, szUser, s, &dwSIDLen, szDomain, &dwDomainLen, &snu);

	if(b == FALSE) {
		return false;
	}


	//Login

	b = ::LogonUser((char*)szUser, szDomain, (char*)szPass, LOGON32_LOGON_NETWORK, LOGON32_PROVIDER_DEFAULT, &hUser);

	if(b == FALSE) {
		return false;
	}

	b = ::ImpersonateLoggedOnUser(hUser);

	if(b == FALSE) {
		return false;
	}


	bLoggedOn = true;
	
	return true;
}


/*
	Returns an instance of a dav command class
*/

CDavCmd* CDavCmdDispatcher::GetCommandInstance()
{
	CDavCmd* pCmd = NULL;

	switch(cmd) {

	case PROPFIND:
		pCmd = new CPropFindCmd(this,pCtx);
		break;

	case MKCOL:
		pCmd = new CMkColCmd(this,pCtx);
		break;

	case DEL:
		pCmd = new CDeleteCmd(this,pCtx);
		break;

	case COPY:
		pCmd = new CCopyCmd(this,pCtx);
		break;

	case MOVE:
		pCmd = new CMoveCmd(this,pCtx);
		break;

	}

	return pCmd;
}
