// DAVFILTER.CPP - Implementation file for your Internet Server
//    DavFilter

/*
 *	Martin Hallerdal, marty@home.se
 *	2000-05-11
 *
 *	Please feel free to experiment with the code. Just don't use it for commercial purposes.
 */

#include "stdafx.h"
#include "DavFilter.h"
#include "RequestInfo.h"
#include "iads.h"
#include "iiis.h"

///////////////////////////////////////////////////////////////////////
// The one and only CWinApp object
// NOTE: You may remove this object if you alter your project to no
// longer use MFC in a DLL.

CWinApp theApp;

FILE* f;

///////////////////////////////////////////////////////////////////////
// The one and only CDavFilter object

CDavFilter theFilter;


///////////////////////////////////////////////////////////////////////
// CDavFilter implementation

CDavFilter::CDavFilter()
{
}

CDavFilter::~CDavFilter()
{
}

BOOL CDavFilter::GetFilterVersion(PHTTP_FILTER_VERSION pVer)
{
	// Call default implementation for initialization
	CHttpFilter::GetFilterVersion(pVer);

	// Clear the flags set by base class
	pVer->dwFlags &= ~SF_NOTIFY_ORDER_MASK;

	// Set the flags we are interested in
	pVer->dwFlags	|= SF_NOTIFY_ORDER_MEDIUM 
					| SF_NOTIFY_SECURE_PORT
					| SF_NOTIFY_NONSECURE_PORT
					| SF_NOTIFY_PREPROC_HEADERS
					| SF_NOTIFY_READ_RAW_DATA
					| SF_NOTIFY_URL_MAP
					| SF_NOTIFY_END_OF_NET_SESSION
					| SF_NOTIFY_SEND_RAW_DATA;

	//Open debug file

	f = fopen("c:\\t\\filterlog.txt", "ab");

	// Load description string
	TCHAR sz[SF_MAX_FILTER_DESC_LEN+1];
	ISAPIVERIFY(::LoadString(AfxGetResourceHandle(),
			IDS_FILTER, sz, SF_MAX_FILTER_DESC_LEN));
	_tcscpy(pVer->lpszFilterDesc, sz);
	return TRUE;
}


DWORD CDavFilter::OnPreprocHeaders(CHttpFilterContext* pCtxt,
	PHTTP_FILTER_PREPROC_HEADERS pHeaderInfo)
{

	CRequestInfo* pReq = (CRequestInfo*)pCtxt->m_pFC->pFilterContext;

	if(pReq) {
		return pReq->OnPreprocHeaders(pCtxt, pHeaderInfo);
	}

	return SF_STATUS_REQ_NEXT_NOTIFICATION;
}

/*
	This will be the penultimate call in our filter for a request. 
	OnPreProcHeaders will tell us if we have more data waiting in which
	case we will make a forced read.
*/

DWORD CDavFilter::OnUrlMap(CHttpFilterContext* pCtxt,
	PHTTP_FILTER_URL_MAP pMapInfo)
{

	CRequestInfo* pReq = (CRequestInfo*)pCtxt->m_pFC->pFilterContext;

	if(pReq) {
		return pReq->OnUrlMap(pCtxt, pMapInfo);
	}

	return SF_STATUS_REQ_NEXT_NOTIFICATION;
}


/*
	Called when we receive raw data. We quit if we should ignore this
	request. Otherwise, copy the data we receive and if it is a forced
	read then we execute the command.	
*/

DWORD CDavFilter::OnReadRawData(CHttpFilterContext* pCtxt,	
	PHTTP_FILTER_RAW_DATA pRawData)
{
	
	CRequestInfo* pReq = (CRequestInfo*)pCtxt->m_pFC->pFilterContext;
	
	//Create a new request object if this is a new session

	if(pReq == NULL) {
		pReq = new CRequestInfo();
		pCtxt->m_pFC->pFilterContext = pReq;
	}

	return pReq->OnReadRawData(pCtxt, pRawData);


}


DWORD CDavFilter::OnSendRawData(CHttpFilterContext* pCtxt,	
								PHTTP_FILTER_RAW_DATA pRawData) {
	
	CRequestInfo* pReq = (CRequestInfo*)pCtxt->m_pFC->pFilterContext;
	
	if(pReq) {
		return pReq->OnSendRawData(pCtxt, pRawData);
	}

	return SF_STATUS_REQ_NEXT_NOTIFICATION;
}

DWORD CDavFilter::OnEndOfNetSession(CHttpFilterContext* pCtxt)
{
	if(pCtxt->m_pFC->pFilterContext) {
		delete (CRequestInfo*)pCtxt->m_pFC->pFilterContext;
		pCtxt->m_pFC->pFilterContext = NULL;
	}

	return SF_STATUS_REQ_NEXT_NOTIFICATION;
}

// Do not edit the following lines, which are needed by ClassWizard.
#if 0
BEGIN_MESSAGE_MAP(CDavFilter, CHttpFilter)
	//{{AFX_MSG_MAP(CDavFilter)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()
#endif	// 0

///////////////////////////////////////////////////////////////////////
// If your extension will not use MFC, you'll need this code to make
// sure the extension objects can find the resource handle for the
// module.  If you convert your extension to not be dependent on MFC,
// remove the comments arounn the following AfxGetResourceHandle()
// and DllMain() functions, as well as the g_hInstance global.

/****

static HINSTANCE g_hInstance;

HINSTANCE AFXISAPI AfxGetResourceHandle()
{
	return g_hInstance;
}

BOOL WINAPI DllMain(HINSTANCE hInst, ULONG ulReason,
					LPVOID lpReserved)
{
	if (ulReason == DLL_PROCESS_ATTACH)
	{
		g_hInstance = hInst;
	}

	return TRUE;
}

****/
