// DeleteCmd.cpp: implementation of the CDeleteCmd class.
//
//////////////////////////////////////////////////////////////////////

/*
 *	Martin Hallerdal, marty@home.se
 *	2000-05-11
 *
 *	Please feel free to experiment with the code. Just don't use it for commercial purposes.
 */


#include "stdafx.h"
#include "DavFilter.h"
#include "DeleteCmd.h"

#include "XMLBody.h"
#include "HttpUtil.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CDeleteCmd::CDeleteCmd(CDavCmdDispatcher* disp, CHttpFilterContext* pCtx) :  CDavCmd(disp, pCtx)
{

}

CDeleteCmd::~CDeleteCmd()
{

}

/*
	Implements the DELETE command	
*/

bool CDeleteCmd::Execute() {

	//204 No Content is the default success code

	resp.SetStatus(204);

	DWORD dwLen = strlen(pDisp->szPhys);
	if(pDisp->szPhys[dwLen-1] == '\\') {
		pDisp->szPhys[dwLen-1] = '\0';
	}

	strcpy(szURI, pDisp->szURI);

	DeleteResource(pDisp->szPhys, true);

	if(bDidInit) {
		doc->documentElement = elemRoot;

		CXMLBody* pResp = new CXMLBody;
		pResp->SetXMLDocument(doc);

		_bstr_t ba = doc->xml;
		char* p = (char*)ba;

		resp.pBody = pResp;
	}
	



	resp.SendResponse(pCtx);

	return true;
}

/*
	Deletes a resource and any child resources.
	Returns true if the resource and its children are deleted.
*/

bool CDeleteCmd::DeleteResource(const char* szPath, bool bFirst) {

	BOOL bDelStat;

	char szBasePath[MAX_PATH]; 
	DWORD dwPathLen = strlen(szPath);
	strncpy(szBasePath, szPath, dwPathLen);
	szBasePath[dwPathLen] = '\0';
	
	DWORD dwURILen = strlen(szURI);
	szURI[dwURILen] = '\0';

	//Get the file attributes

	DWORD dwAttrs = ::GetFileAttributes(szPath);

	if(dwAttrs == 0xFFFFFFFF) {
		//Failed to retrieve properties.

		SetError(bFirst, szURI);

		return false;
	} 


	//Treat directories and files differently
	
	if(dwAttrs & FILE_ATTRIBUTE_DIRECTORY) {
		
		//Directory. Traverese it and delete members
		
		bool bSucceeded = true;
		
		strcat(szBasePath, "\\*.*");
	
		szURI[dwURILen+1] = '\0';
		szURI[dwURILen] = '/';


		WIN32_FIND_DATA fdata;
		BOOL bFind;			
		
		HANDLE hFind = ::FindFirstFile(szBasePath, &fdata);
		
		if(hFind != INVALID_HANDLE_VALUE) {
			
			//Loop through the files
			
			while(true) {

				if(strcmp(fdata.cFileName, ".") && strcmp(fdata.cFileName, "..")) {
					strcat(szURI, fdata.cFileName);

					szBasePath[dwPathLen+1] = '\0';
					strcat(szBasePath, fdata.cFileName);
				
					bSucceeded &= DeleteResource(szBasePath, false);

					szURI[dwURILen+1] = '\0';
				}
				

				//Get the next file
				
				bFind = ::FindNextFile(hFind, &fdata);
				
				if(bFind == FALSE) {
					if(GetLastError() == ERROR_NO_MORE_FILES) {
						break;
					}
				}
			}
			
			::FindClose(hFind);	
		}


		//If nothing went wrong then try to delete the directory itself
		
		if(bSucceeded) {

			bDelStat = ::RemoveDirectory(szPath);
		
			if(bDelStat == FALSE) {
				SetError(bFirst, szURI);
				return false;
			}

			return true;
		}

	} else {
		
		//File. Delete it.
		
		bDelStat = ::DeleteFile(szPath);
		
		if(bDelStat == FALSE) {
			SetError(bFirst, szURI);
			return false;
		}

		return true;
	}
		
	
	
	return false;
}

