// Headers.cpp: implementation of the CHeaders class.
//
//////////////////////////////////////////////////////////////////////


/*
 *	Martin Hallerdal, marty@home.se
 *	2000-05-11
 *
 *	Please feel free to experiment with the code. Just don't use it for commercial purposes.
 */

#include "stdafx.h"
#include "Headers.h"
#include "Base64String.h"
#include "HttpConst.h"
#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////


CHeaders::CHeaders() : szBasicUser(NULL), szBasicPassword(NULL)
{

}

CHeaders::~CHeaders() {

	Clear();
}

/*
	Clears all the headers
*/

void CHeaders::Clear() {
	HEADERS::iterator it;

	for(it = hdrs.begin(); it != hdrs.end(); it++) {
		delete (*it).first;
		delete (*it).second;
	}

	SAFE_FREE_ARRAY(szBasicUser)
	SAFE_FREE_ARRAY(szBasicPassword)

}

/*
	Parses header data.
	
	  pcData		The data which contains the header (Not \0 terminated)
	  dwInputLen	Length of pcData
	  plLeftOver	Number of bytes that wasn't header data
	

	*STATIC*
*/

CHeaders* CHeaders::ParseHeaders(char *pcData, DWORD dwInputLen, long* plLeftOver) {
	
	if(pcData == NULL || plLeftOver == NULL) {
		return NULL;
	}


	// It's much nicer to have a NULL-terminated string. This doesn't incur much overhead I think... 

	char* szData = new char[dwInputLen+1];
	strncpy(szData, pcData, dwInputLen);
	szData[dwInputLen] = '\0';
	

	char* szLoc = szData;
	char* szPrevious = szData;

	char* szValue;

	int iKeyLength;
	int iValueLength;

	//The first line is the request line with the method, uri, and version

	szLoc = strstr(szData, "\r\n");
	
	if(szLoc == NULL) {
		return NULL;
	}
	
	szLoc += 2;
	szPrevious = szLoc;

	//If we can't find \r\n\r\n this isn't all the headers;
	
	char* szDelim = strstr(szLoc, "\r\n\r\n");

	if(szDelim == NULL) {
		return NULL;
	}
			
	//Number of leftover bytes
	
	*plLeftOver = dwInputLen - ((szDelim+4)-szData);

	CHeaders* hdrs = new CHeaders();

	while(true) {

		//Find next 

		szLoc = strstr(szPrevious, ":");

		if(szLoc == NULL) {
			break;
		}

		iKeyLength = szLoc - szPrevious;

		szLoc++;	//Walk past the ':'

		//Go forward to where the real data is

		while(*szLoc == ' ' && szLoc != '\0') {
			szLoc++;
		}

		szValue = szLoc;
		
		//Find the newline


		szLoc = strstr(szLoc, "\r\n");

		if(szLoc == NULL) {
			break;
		}
		
		iValueLength = szLoc - szValue;

		hdrs->SetHeader(szPrevious, iKeyLength, szValue, iValueLength);

		//Quit if we are at the end

		if(szLoc == szDelim) {
			break;
		}

		szPrevious = szLoc + 2;	//Walk past the linebreak

	}

	delete[] szData;
		
	hdrs->parseAuthorisation();

	return hdrs;
}

/*
	Sets a header
*/

void CHeaders::SetHeader(char *szKey, int iKeyLength, char *szValue, int iValueLength) {

	if(iKeyLength == -1) {
		iKeyLength = strlen(szKey);
	}

	if(iValueLength == -1) {
		iValueLength = strlen(szValue);
	}

	char* szNyKey = new char[iKeyLength+1];
	char* szNyVal = new char[iValueLength+1];

	strncpy(szNyKey, szKey, iKeyLength);
	strncpy(szNyVal, szValue, iValueLength);

	szNyKey[iKeyLength] = '\0';
	szNyVal[iValueLength] = '\0';

	hdrs.insert(HEADERS::value_type(szNyKey, szNyVal));

}


/*
	Gets a header
*/

const char* CHeaders::GetHeader(const char* szKey) {
	
	HEADERS::iterator it;

	for(it = hdrs.begin(); it != hdrs.end(); it++) {

		if(_stricmp((*it).first, szKey) == 0) {
			return (const char*)(*it).second;
		}
	}
	
	return NULL;
	
}


/*
	Searches the headers for any authorisation information that client may have sent
*/

void CHeaders::parseAuthorisation() {

	const char* szAuth = GetHeader("Authorization");

	if(szAuth == NULL) {
		return;
	}

	//szAuth should look something like 'Basic QWxhZGRpbjpvcGVuIHNlc2FtZQ=='

	//We only support basic authentication

	while(*szAuth == ' ' && szAuth != '\0') {
		szAuth++;
	}

	if(_strnicmp(szAuth, "Basic", 5) == 0) {

		szAuth += 5;

		while(*szAuth == ' ' && szAuth != '\0') {
			szAuth++;
		}

		CBase64String encStr(szAuth);

		//Parse user and password

		char* szDec = (char*)encStr;
		char* szPtr = strchr(szDec, ':');

		if(szPtr != NULL) {
			
			//Copy user

			szBasicUser = new char[(szPtr-szDec) + 1];
			strncpy(szBasicUser, szDec, (szPtr-szDec));
			szBasicUser[(szPtr-szDec)] = '\0';


			//Walk past ':'

			szPtr++;

			//Copy password
			
			UINT iPassLength = strlen(szPtr);

			szBasicPassword = new char[iPassLength+1];
			strcpy(szBasicPassword, szPtr);
			szBasicPassword[iPassLength] = '\0';
		}

	}
}


const char* CHeaders::GetBasicUser() const {
	return szBasicUser;
}
const char* CHeaders::GetBasicPassword() const {
	return szBasicPassword;
}
