#include "StdAfx.h"
#include "HttpUtil.h"

/*
 *	Martin Hallerdal, marty@home.se
 *	2000-05-11
 *
 *	Please feel free to experiment with the code. Just don't use it for commercial purposes.
 */


#include <winerror.h>
#include <math.h>
#include <atlbase.h>
#include <comdef.h>

#include "iads.h"
#include "adshlp.h"
#include "IIIS.h"


#define IS_GOOD_URL_CHAR(c) (isalnum((c)) || (c) == '+' || (c) == '*' || (c) == '/' || (c) == '.' || (c) == '_' || (c) == '@')

/*
	Encodes a string that makes it suitable for a URL
	
	NOTE: Not threadsafe
*/
void URLEncode(char* szEnc, DWORD dwBuffLen) {

	char* szCurr = szEnc;
	DWORD dwLen = strlen(szEnc);
	int iBadChars = 0;

	//Count bad chars

	for(DWORD i=0; i < dwLen; i++) {
		if(!IS_GOOD_URL_CHAR(szEnc[i])) {
			iBadChars++;
		}
	}

	//Calculate new length

	DWORD dwNewLen = dwLen + 2*iBadChars;

	if(dwNewLen >= dwBuffLen) {
		return;
	}

	char* szNy = new char[dwNewLen+1];
	int iDestPos = 0;

	for(i=0; i < dwLen;i++) {

		if(!IS_GOOD_URL_CHAR(szEnc[i])) {
			
			sprintf(szNy+iDestPos, "%%%X", szEnc[i]);

			iDestPos += 3;

		} else {
			szNy[iDestPos] = szEnc[i];
			iDestPos++;
		}
	}

	szNy[iDestPos] = '\0';

	strcpy(szEnc, szNy);
	
	delete[] szNy;

}

/*
	Decodes a string containing a hex number to a number
	Sort of atoi but with hex instead

	NOTE: Not threadsafe
*/

DWORD DecodeHexString(char* pbString, int len) {

	int iPVal;
	DWORD dwRet = 0;

	for(int i=0; i < len; i++) {
		
		if(isdigit(pbString[i])) {
			iPVal = pbString[i] - '0';

		} else if(isalpha(pbString[i])) {
			iPVal = tolower(pbString[i]) - 'a' + 10;
		}

		dwRet += iPVal * (int)pow((double)16, (double)(len-i-1));
	}

	return dwRet;
}

/*
	Decodes a URL encoded string

	NOTE: Not threadsafe
*/

void URLDecode(char* szEnc) {

	DWORD dwLen = strlen(szEnc);
	DWORD dwDestPos = 0;

	for(DWORD i=0; i < dwLen;) {
		
		if(szEnc[i] == '%') {
			szEnc[dwDestPos] = (char)DecodeHexString(szEnc+i+1, 2);
			i += 3;

		} else {
			szEnc[dwDestPos] = szEnc[i];
			i++;
		}

		dwDestPos++;
	}
	
	szEnc[dwDestPos] = '\0';
}


/*
	Converts val to a string and also makes sure it contains two digits
*/


inline void Pad(char* szBuff, int val) {
	if(val < 10) {
		sprintf(szBuff, "%d%d", 0, val);
	} else {
		sprintf(szBuff, "%d", val);
	}
}


char* GetStatusLineFromCode(DWORD dwStatus) {

	char *szStatus = "";

	switch(dwStatus) {

	case 200:
		szStatus = "HTTP/1.1 200 OK\r\n";
		break;
		
	case 201:
		szStatus = "HTTP/1.1 201 Created\r\n";
		break;
		
	case 204:
		szStatus = "HTTP/1.1 204 No Content\r\n";
		break;
		
	case 207:
		szStatus = "HTTP/1.1 207 Multi-Status\r\n";
		break;

	case 401:
		szStatus = "HTTP/1.1 401 Unauthorized\r\n";
		break;

	case 403:
		szStatus = "HTTP/1.1 403 Forbidden\r\n";
		break;
		
	case 404:
		szStatus = "HTTP/1.1 404 Not found\r\n";
		break;
		
	case 405:
		szStatus = "HTTP/1.1 405 Method Not Allowed\r\n";
		break;
		
	case 409:
		szStatus = "HTTP/1.1 409 Conflict\r\n";
		break;
		
	case 412:
		szStatus = "HTTP/1.1 412 Precondition Failed\r\n";
		break;
		
	case 415:
		szStatus = "HTTP/1.1 415 Unsupported Media Type\r\n";
		break;
		
	case 423:
		szStatus = "HTTP/1.1 423 Locked\r\n";
		break;
		
	case 502:
		szStatus = "HTTP/1.1 502 Bad Gateway\r\n";
		break;
		
	case 507:
		szStatus = "HTTP/1.1 507 Insufficient Storage\r\n";
		break;

	}

	return szStatus;
}

/*
	Checks the current windows error code and returns an appropriate HTTP status line
*/

char* GetHTTPStatusLine(DWORD dwError) {

	if(dwError == -1) {
		dwError = ::GetLastError();
	}

	char* szStatus = "";

	switch(dwError) {
	
	case ERROR_SUCCESS:
		return GetStatusLineFromCode(200);
		break;

	case ERROR_FILE_NOT_FOUND:
		return GetStatusLineFromCode(404);
		break;

	case ERROR_PATH_NOT_FOUND:
		return GetStatusLineFromCode(404);
		break;

	case ERROR_ACCESS_DENIED:
		return GetStatusLineFromCode(401);
		break;

	case ERROR_ALREADY_EXISTS:
		return GetStatusLineFromCode(412);
		break;

	default:
		return GetStatusLineFromCode(404);
		break;
	}
	
	return szStatus;	
}


/*
	Returns a ISO8601 time
	
	NOTE: Not threadsafe
*/

BOOL GetISO8601Time(FILETIME& ft, char* szBuffer) {

	//Offset
	
	static bool didInit = false;
	static LONG bias;

	static char szOff[7];
	
	if(!didInit) {

		didInit = true;

		//Get timezone information

		TIME_ZONE_INFORMATION tz;
		DWORD dwRet = GetTimeZoneInformation(&tz);

		if(dwRet == TIME_ZONE_ID_UNKNOWN || dwRet == TIME_ZONE_ID_INVALID) {
			bias = 0;

			strcpy(szOff, "-00:00");
			szOff[6] = '\0';
		} else {

			bias = tz.Bias;
			char cSign = bias < 0 ? '-' : '+';
			bias = bias < 0 ? -bias : bias;

			int iMinutes = bias % 60;
			int iHours = bias / 60;


			char szMin[3];
			char szHour[3];

			Pad(szMin, iMinutes);
			Pad(szHour, iHours);
			
			sprintf(szOff, "%c%s:%s", cSign, szHour,  szMin);
		}


	}

	//Get the time

	SYSTEMTIME systime;

	BOOL bRetVal = ::FileTimeToSystemTime(&ft, &systime);

	if(!bRetVal) {
		return FALSE;
	}
	
	char szMon[3];
	char szDay[3];
	char szHour[3];
	char szMinute[3];
	char szSecond[3];

	Pad(szMon, systime.wMonth);
	Pad(szDay, systime.wDay);
	Pad(szHour, systime.wHour);
	Pad(szMinute, systime.wMinute);
	Pad(szSecond, systime.wSecond);

	sprintf(szBuffer, "%d-%s-%sT%s:%s:%s%s", systime.wYear, szMon, szDay, szHour, szMinute, szSecond, szOff);
	
	return TRUE;
}

/*
	Returns a mime type for a file
	
	NOTE: Not threadsafe
*/

BOOL GetMimeType(char* szFile, char* szBuff) {

	//Find extension

	char* szExt = szFile;

	while(*szExt != '\0' && *szExt != '.') {
		szExt++;
	}



	//If we found it, look in the array
	
	if(*szExt != '\0') {
		
		_bstr_t bstrExt(szExt);
		
		static CComPtr<IADs> adMimeMap;
		static bool bDidInit = false;
		static bool bGoodObject = false;
		static VARIANT vMimeMap;
		HRESULT hr;


		//Look up the mimemap object that IIS has

		if(!bDidInit) {
			
			bDidInit = true;
			
			hr = ::ADsGetObject(L"IIS://LocalHost/MimeMap", IID_IADs, (void**)&adMimeMap);
			
			if(SUCCEEDED(hr)) {
				//Get the "MimeMap" property
				
				
				::VariantInit(&vMimeMap);
				
				hr = adMimeMap->GetEx(L"MimeMap", &vMimeMap);
				
				if(SUCCEEDED(hr)) {
					
					//It should be an array
					
					if(vMimeMap.vt & VT_ARRAY) {
						bGoodObject = true;
					}
				}
			}
		}
			
			//Only proceed if we have a valid reference
		
		if(bGoodObject) {
			
			
			
			//Declare some nifty variables
			
			long lIdx;
			long lHigh;
			
			_bstr_t bstrBuff;
			BSTR bstrRaw;
			
			
			CComQIPtr<IISMimeType> MimeType;					
			
			VARIANT vElem;
			::VariantInit(&vElem);
			
			SAFEARRAY* pArrMimeMap = vMimeMap.parray;
			
			//Get the upper and lower bounds of the array
			
			::SafeArrayGetLBound(pArrMimeMap, 1, &lIdx);
			::SafeArrayGetUBound(pArrMimeMap, 1, &lHigh);
			
			
			//Loop through the array
			
			while(lIdx <= lHigh) {
				
				hr = ::SafeArrayGetElement(pArrMimeMap, &lIdx , &vElem);
				
				if(SUCCEEDED(hr)) {
					if(vElem.vt & VT_DISPATCH) {
						
						//Each element in the array should be of type IISMimeType
						
						MimeType = vElem.pdispVal;
						
						if(MimeType != NULL) {
							
							MimeType->get_Extension(&bstrRaw);
							bstrBuff = bstrRaw;
							
							//Check if the extension matches
							
							if(bstrExt == bstrBuff) {
								MimeType->get_MimeType(&bstrRaw);
								bstrBuff = bstrRaw;
								strcpy(szBuff, (char*)bstrBuff);
								return TRUE;
							}
							
						}
						
						MimeType.Release();
					}
				}
				
				lIdx++;
				
			}	// Loop through elements
		
		
		
		} //bGoodObject
		
		
		
	}	//szExt != \0
	
	*szBuff = '\0';

	return FALSE;
}

/*
	HELPER

	Checks if there exists a virtual dir in szADSPath called szName

	NOTE: Not threadsafe
*/

bool ExistsVirtualDir(const char* szADSPath, const char* szName) {

	HRESULT hr;

	_bstr_t bstrTemp;

	CComPtr<IEnumVARIANT>	pEnum;
	CComPtr<IADsContainer>	pCont;
	CComPtr<IADs>			pADs;
	CComPtr<IUnknown>		pUnk;

	VARIANT			var;
	IDispatch*		pDisp;
	BSTR			bstr = NULL;
	ULONG			lFetch;

 
	bstrTemp = szADSPath;
	

	hr = ::ADsGetObject(bstrTemp, 
                        IID_IADsContainer, (void**) &pCont);

	hr = pCont->get__NewEnum(&pUnk);

	
	hr = pUnk->QueryInterface(IID_IEnumVARIANT, (void**) &pEnum);
	
	::VariantInit(&var);
	
	// Now Enumerate
	
	hr = pEnum->Next(1, &var, &lFetch);
	while(hr == S_OK)
	{
		if (lFetch == 1)
		{
			pDisp = V_DISPATCH(&var);
			hr = pDisp->QueryInterface(IID_IADs, (void**)&pADs); 
			pDisp->Release();
			V_DISPATCH(&var) = NULL;
			
			if(SUCCEEDED(hr)) {
				
				pADs->get_Class(&bstr);
				bstrTemp = bstr;
				
				if(!strcmp((char*)bstrTemp, "IIsWebVirtualDir")) {
					//Ok, its a webserver
					
					pADs->get_Name(&bstr);
					bstrTemp = bstr;
					
					if(!_stricmp((char*)bstrTemp, szName)) {
						return true;
					}

				}
				
				
				pADs.Release();
			}
		}
		::VariantClear(&var);
		hr = pEnum->Next(1, &var, &lFetch);
	};
	
	if(bstr) {
		SysFreeString(bstr);
	}

	return false;
}


/*
	HELPER
  
	Takes a URI and returns a physical path

	NOTE: Not threadsafe
*/

bool GetPhysicalPath(const char* szURI, char* szOut) {

	if(*szURI != '/') {
		return false;
	}

	//Find the next dir or null

	char cOld;
	char* szEnd = strchr(szURI+1, '/');

	if(szEnd) {
		cOld = *szEnd;
		*szEnd = '\0';
	}

	//Walk past first /

	szURI++;

	//Ok, now we have a name, check if we have a virtual directory which has that name

	HRESULT hr;

	_bstr_t bstrTemp;
	VARIANT vPath;

	CComPtr<IEnumVARIANT>	pEnum;
	CComPtr<IADsContainer>	pCont;
	CComPtr<IADs>			pADs;
	CComPtr<IADs>			pADVirs;
	CComPtr<IUnknown>		pUnk;

	bool bFoundIt = false;
	

	VARIANT			var;
	IDispatch*		pDisp;
	BSTR			bstr;
	ULONG			lFetch;

 
	char szBasePath[256] = "IIS://LocalHost/W3SVC";
	DWORD dwLen = strlen(szBasePath);

	bstrTemp = szBasePath;

	//Create object

	hr = ::ADsGetObject(bstrTemp, IID_IADsContainer, (void**) &pCont);

	if(FAILED(hr)) {
		return false;
	}

	hr = pCont->get__NewEnum(&pUnk);

	pCont.Release();
	
	hr = pUnk->QueryInterface(IID_IEnumVARIANT, (void**) &pEnum);

	pUnk.Release();
	
	::VariantInit(&var);
	
	// Enumerate all keys under the web service
	
	hr = pEnum->Next(1, &var, &lFetch);
	while(hr == S_OK && !bFoundIt)
	{
		if (lFetch == 1)
		{
			//Get a IDispatch binding

			pDisp = V_DISPATCH(&var);
			hr = pDisp->QueryInterface(IID_IADs, (void**)&pADs); 
			pDisp->Release();
			V_DISPATCH(&var) = NULL;
			
			if(SUCCEEDED(hr)) {
				
				pADs->get_Class(&bstr);
				bstrTemp = bstr;
				
				//Check if key is a web server

				if(!strcmp((char*)bstrTemp, "IIsWebServer")) {
					
					
					pADs->get_Name(&bstr);
					bstrTemp = bstr;

					strcat(szBasePath, "/");
					strcat(szBasePath, (char*)bstrTemp);
					strcat(szBasePath, "/ROOT");
					
					if(ExistsVirtualDir(szBasePath, szURI)) {
						//Ok, convert every / to a \ and append the stuff

						strcat(szBasePath, "/");
						strcat(szBasePath, szURI);
						
						bFoundIt = true;
						
					}
					
					if(!bFoundIt) {
						szBasePath[dwLen] = '\0';
					}
				}
				
				
				pADs.Release();
			}
		}
		::VariantClear(&var);
		hr = pEnum->Next(1, &var, &lFetch);
	};

	pEnum.Release();

	if(!bFoundIt) {
		//Then it is root. Assume first web service
		strcat(szBasePath, "/1/ROOT");

		if(szEnd) {
			*szEnd = '/';
		}

		szEnd = (char*)szURI-1;	//We want the leading /
	}
	
	
	//Find the physical path of the virtual directory identified by szBasePath
	
	bstrTemp = szBasePath;
	
	hr = ::ADsGetObject(bstrTemp, IID_IADs, (void**) &pADVirs);
	
	if(SUCCEEDED(hr)) {
		hr = pADVirs->Get(L"Path", &vPath);
		
		pADVirs.Release();
		
		if(SUCCEEDED(hr)) {
			bFoundIt = true;
			bstrTemp = vPath.bstrVal;
			strcpy(szOut, (char*)bstrTemp);
			DWORD dwLen = strlen(szOut);
			
			szOut[dwLen] = '\\';
			szOut[dwLen+1] = '\0';
			
			//Paste the rest of the URI
			
			if(szEnd) {
				strcat(szOut, szEnd+1);
			}
			
			char* szCurr = szOut+dwLen;
			
			while(*szCurr) {
				if(*szCurr == '/') {
					*szCurr = '\\';
				}
				
				szCurr++;
			}

			//Remove a trailing '\'

			if(*(szCurr-1) == '\\') {
				*(szCurr-1) = '\0';
			}
		}
	}

	//Restore

	if(szEnd) {
		*szEnd = '/';
	}

	SysFreeString(bstr);	

	return bFoundIt;
}

/*
	Maps a URL to a physical directory
	
	NOTE: Not threadsafe
*/

bool MapPath(const char* szURL, char* szOut, CHttpFilterContext* pCtx) {

	BOOL bRet=TRUE;
	
	DWORD dwSize = 256;
	char szBuff[256];

	//Get server name

	bRet = pCtx->GetServerVariable("SERVER_NAME", szBuff, &dwSize);

	if(bRet == FALSE) {
		return false;
	}

	//Find server portion

	char* szServ = strstr(szURL, "://");
	
	if(szServ == NULL) {
		return false;
	}

	szServ += 3;
	
	//Find the port or the /

	char* szPort = strchr(szServ, ':');
	char* szSlash = strchr(szServ, '/');

	if(!szPort && !szSlash) {
		return false;
	}

	char* szDelim = NULL;
	
	if(szPort && !szSlash) {
		szDelim = szPort;
	} else if(!szPort && szSlash) {
		szDelim = szSlash;
	} else {
		szDelim = (szPort < szSlash) ? szPort : szSlash;
	}

	//Check if they have the right server

	char cOld = *szDelim;
	*szDelim = '\0';

	if(strcmp(szServ, szBuff)) {
		return false;
	}

	//Find the first slash

	*szDelim = cOld;

	if(cOld != '/') {
		szDelim = strchr(szDelim, '/');

		if(szDelim == NULL) {
			return false;
		}
	}

	//We might wanna check for a query string but I feel really tired right now.

	return GetPhysicalPath(szDelim, szOut);
}


