// MoveCmd.cpp: implementation of the CMoveCmd class.
//
//////////////////////////////////////////////////////////////////////

/*
 *	Martin Hallerdal, marty@home.se
 *	2000-05-11
 *
 *	Please feel free to experiment with the code. Just don't use it for commercial purposes.
 */


#include "stdafx.h"
#include "DavFilter.h"
#include "MoveCmd.h"
#include "HttpUtil.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CMoveCmd::CMoveCmd(CDavCmdDispatcher* disp, CHttpFilterContext* pCtx) : CDavCmd(disp, pCtx)
{

}

CMoveCmd::~CMoveCmd()
{

}

/*
	Performs a MOVE command

	NOTE: Ignores any depth header
*/

bool CMoveCmd::Execute() {
	
	bool bExist = true;
	bool bDestExist = true;
	BOOL bSucc;

	//Get overwrite

	const char* szOW = pDisp->pHdrs->GetHeader("Overwrite");
	
	bool bOverwrite = szOW ? ((*szOW == 'T') ? true : false) : false; 
	
	//Get destination

	const char* szDest = pDisp->pHdrs->GetHeader("Destination");
	URLDecode((char*)szDest);

	char szPhysDest[MAX_PATH];
	do {
		
		if(!MapPath(szDest, szPhysDest, pCtx)) {
			resp.SetStatus(502);
			break;
		} 

		//Check if dest and sourc are the same

		char* szDestURI = strstr(szDest, "://");

		if(szDestURI) {
			szDestURI = strchr(szDestURI+3, '/');
		
			if(szDestURI) {
				if(!_stricmp(szDestURI, pDisp->szURI)) {
					resp.SetStatus(403);
					break;
				}
			}
	
		}
		
		//Check if destination and source exist
		
		DWORD dwAttsDst = ::GetFileAttributes(szPhysDest);
		if(dwAttsDst == 0xFFFFFFFF) {
			if(::GetLastError() == ERROR_FILE_NOT_FOUND || ::GetLastError() == ERROR_PATH_NOT_FOUND) {
				bDestExist = false;
			}
		}
		
		dwAttsDst = ::GetFileAttributes(pDisp->szPhys);
		if(dwAttsDst == 0xFFFFFFFF) {
			if(::GetLastError() == ERROR_FILE_NOT_FOUND || ::GetLastError() == ERROR_PATH_NOT_FOUND) {
				bExist = false;
			}
		}
		
		if(!bExist) {
			resp.SetStatus(404);
			break;
		} 
		
		//Does source parent exist?
		
		if(!AncestorExist(szPhysDest)) {
			resp.SetStatus(409);
			break;		
		} 
		
		//If exist and not overwrite -> fail	
		
		if(bDestExist && !bOverwrite) {
			resp.SetStatus(412);
			break;
		} 
		
		//Delete destination if exist
		
		if(bDestExist) {
			//Delete...
			if(!DeleteFileObject(szPhysDest)) {
				SetError(true, pDisp->szURI);
				break;
			}
		}
		
		//Move
		
		bSucc = ::MoveFile(pDisp->szPhys,szPhysDest);
		
		//Failed?
		
		if(!bSucc) {
			SetError(true, pDisp->szURI);
		} else {
			if(!bDestExist) {
				resp.SetStatus(201);
			} else {
				resp.SetStatus(204);
			}
		}
		
	} while(false);

	resp.SendResponse(pCtx);
	
	return true;
}


/*
	Deletes a resource and any child resources.
	Returns true if the resource and its children are deleted.

	A stripped version of CDelete::DeleteResource
*/

bool CMoveCmd::DeleteFileObject(const char* szPath) {

	BOOL bDelStat;
	
	char szBasePath[MAX_PATH]; 
	DWORD dwPathLen = strlen(szPath);
	strncpy(szBasePath, szPath, dwPathLen);
	szBasePath[dwPathLen] = '\0';

	//Get file attributes

	DWORD dwAttrs = ::GetFileAttributes(szPath);

	if(dwAttrs == 0xFFFFFFFF) {
		//Failed to retrieve properties.

		return false;
	} 

	//Treat directories and files differently
	
	if(dwAttrs & FILE_ATTRIBUTE_DIRECTORY) {
		
		//Directory. Traverese it and delete members
		
		bool bSucceeded = true;
		
		strcat(szBasePath, "\\*.*");
	
		WIN32_FIND_DATA fdata;
		BOOL bFind;			
		
		HANDLE hFind = ::FindFirstFile(szBasePath, &fdata);
		
		if(hFind != INVALID_HANDLE_VALUE) {
			
			//Loop through the files
			
			while(true) {

				if(strcmp(fdata.cFileName, ".") && strcmp(fdata.cFileName, "..")) {

					szBasePath[dwPathLen+1] = '\0';
					strcat(szBasePath, fdata.cFileName);
				
					bSucceeded &= DeleteFileObject(szBasePath);
				}
				

				//Get the next file
				
				bFind = ::FindNextFile(hFind, &fdata);
				
				if(bFind == FALSE) {
					if(GetLastError() == ERROR_NO_MORE_FILES) {
						break;
					}
				}
			}
			
			::FindClose(hFind);	
		}


		//If nothing went wrong then try to delete the directory itself
		
		if(bSucceeded) {

			bDelStat = ::RemoveDirectory(szPath);
		
			if(bDelStat == FALSE) {
				return false;
			}

			return true;
		}

	} else {
		
		//File. Delete it.
		
		bDelStat = ::DeleteFile(szPath);
		
		if(bDelStat == FALSE) {
			return false;
		}

		return true;
	}
		
	
	
	return false;
}
