//
// 
// RTFEX.H
// 
// Copyright (c) 1998,99 On Time
// http://www.on-time.com
// 
// C Exception Handling Library Header File
// 
// 

// Define XWIN32 to use the x86/Win32 alternate implementation
// for saving/restoring the context.
// If XWIN32 is not defined, the portable method with setjmp/longjmp
// is used.

#ifdef XWIN32

   typedef struct {
      unsigned long esi, edi, ebx, ret, ebp, esp;
   } XContext;

   int  __stdcall XSaveContext(XContext * C);
   void __stdcall XRestoreContext(XContext * C, int Value); 

   #define XCONTEXT              XContext
   #define XSAVECONTEXT(C)       XSaveContext(C)
   #define XRESTORECONTEXT(C, V) XRestoreContext(C, V)

#else // setjmp()/longjmp() version

   #include <setjmp.h>
   #define XCONTEXT              jmp_buf
   #define XSAVECONTEXT(C)       setjmp(*(C))
   #define XRESTORECONTEXT(C, V) longjmp(*(C), V)

#endif


// data needed for each XTRY block

typedef struct _XRecord {
   struct _XRecord * Next;
   XCONTEXT          Context;
   int               XValue;
   int               IsHandled;
   enum { XCode, 
          XHandling, 
          XFinally } State;
} XRecord;


// special exception codes

#define XNOEX     0      // no exception
#define XCODE     XNOEX  // value for the code block
#define XFINALLY  -1     // value for the finally block


// internal functions

void XLinkExceptionRecord(XRecord * XD);
int  XUnLinkExceptionRecordCanReturn(XRecord * XD);
int  XUnLinkExceptionRecordNoReturn(XRecord * XD);


// API macros and functions

#define XTRY    while (1) { XRecord XData; XLinkExceptionRecord(&XData); switch (XSAVECONTEXT(&XData.Context)) {
#define XEND    } { int x = XData.XValue; if (XUnLinkExceptionRecordCanReturn(&XData)) return XData.XValue; else if (x <= 0) break; } }
#define XENDX   } if (XUnLinkExceptionRecordNoReturn(&XData) == 0) break; }
#define XVALUE  XData.XValue

void XRaise(int XValue);
void XReExecute(void);
void XHandled(void);
