//
// SID.h
//
// Created by Marcelo Calbucci
//
// April/2000
//
// Compile with _WIN32_WINNT=0x0500 and _UNICODE defined

#ifndef __SID_H__
#define __SID_H__

#include <string>

using namespace std;

enum eWellKnownSids {
    WKS_Self,
    WKS_Everyone,
    WKS_AuthenticatedUsers,
    WKS_CreatorOwner,
    WKS_Administrators,
    WKS_AccountOperators,
    WKS_BackupOperators,
    WKS_Guests,
    WKS_PrintOperators,
    WKS_Users,

    WKS_CurrentProcessSid,
    WKS_CurrentThreadSid
};

class CSid
{
public:
    CSid();
    CSid(const CSid & rhs);
    CSid(eWellKnownSids);
    CSid(const PSID pSid);
    
    ~CSid();
    
    CSid &      operator=(const CSid & rhs);
    BOOL        operator==(const CSid & rhs) const;
    BOOL        operator!=(const CSid & rhs) const;
    
    // Set
    void	SetSid(const CSid & rhs);
    DWORD	SetSid(const PSID pSid);
    DWORD	SetSidFromString(const wstring & wsStringSid);
    DWORD       SetSidFromName(const wstring & wsAccountName, LPCTSTR pszServer = NULL);
    DWORD       SetSid(eWellKnownSids eSid);       
    
    // Get
    const PSID	GetSid() const { return m_pSid; }
    PSID	GetSidCopy(int iMinSpace = 0) const;
    DWORD       GetStringSid(wstring & wsSid) const;
    
    // Misc
    DWORD       LengthRequired() const;
    BOOL        IsSamePrefix(const CSid & rhs) const;
    void	Clear();
    
    static void DeleteSid(PSID pSid);
    static BOOL GetAccountName(const PSID pSid, wstring & wsAccountName, LPCTSTR lpszServer = NULL);
    static PSID GetAccountSid(const wstring & wsAccountName, LPCTSTR lpszServer = NULL); // Call DeleteSid() after use

protected:
    DWORD       GetTokenSid(eWellKnownSids eSid);
    
protected:
    BOOL	m_fUseLocalFree; // TRUE if the content of m_pSid should be freed with LocalFree
    PSID	m_pSid;
    
#ifdef _DEBUG
    wstring	m_wsName;
#endif
    
};


#endif