//
// ACE.h
//
// Created by Marcelo Calbucci
//
// April/2000
//

#ifndef __ACE_H__
#define __ACE_H__

#include "windows.h"
#include "sderror.h"
#include "sid.h"

enum eAceType {
	AT_AccessAllowed,
	AT_AccessDenied,
	AT_AuditAccess
};

// ACE Flags, defined at winnt.h and ntseapi.h
// For all types of ACEs:
//      OBJECT_INHERIT_ACE                (0x1)
//      CONTAINER_INHERIT_ACE             (0x2)
//      NO_PROPAGATE_INHERIT_ACE          (0x4)
//      INHERIT_ONLY_ACE                  (0x8)
// Read-only:
//      INHERITED_ACE                     (0x10)
// For Audit Access ACEs only:
//      SUCCESSFUL_ACCESS_ACE_FLAG        (0x40)
//      FAILED_ACCESS_ACE_FLAG            (0x80)


extern GUID gNullGuid;

class CAcl;

class CAce
{
public:
    friend CAcl;

    CAce();
    ~CAce();
    CAce(const CAce & rhs);
    CAce(const LPVOID pAce);
    CAce(eAceType eatType, const PSID pSid, DWORD dwMask, DWORD dwAceFlags = 0, 
         const GUID & guidObjectType = gNullGuid, 
         const GUID & guidInheritedObjectType = gNullGuid);
    CAce(eAceType eatType, const CSid & sid, DWORD dwMask, DWORD dwAceFlags = 0, 
         const GUID & guidObjectType = gNullGuid, 
         const GUID & guidInheritedObjectType = gNullGuid);

    CAce &      operator=(const CAce & rhs);
    BOOL        operator==(const CAce & rhs) const;
    BOOL        operator!=(const CAce & rhs) const;

    BOOL        Contains(const CAce & rhs) const;
    
    // Set
    void	SetAce(const CAce & rhs);
    DWORD	SetAce(const LPVOID pAce);
    
    void	SetType(eAceType eatType);
    DWORD	SetSid(const PSID pSid);
    void	SetSid(const CSid & sid);
    void	SetFlags(DWORD dwAceFlags);
    void	SetMask(DWORD dwMask);
    void	SetObjectType(const GUID & guidObjectType);
    void	SetInheritedObjectType(const GUID & guidInheritedObjectType);
    
    // SetAce
    DWORD	SetAce(eAceType eatType, const PSID pSid, DWORD dwMask, DWORD dwAceFlags = 0, 
                       const GUID & guidObjectType = gNullGuid, 
                       const GUID & guidInheritedObjectType = gNullGuid);
    void	SetAce(eAceType eatType, const CSid & sid, DWORD dwMask, DWORD dwAceFlags = 0, 
                       const GUID & guidObjectType = gNullGuid, 
                       const GUID & guidInheritedObjectType = gNullGuid);
    
    // Get
    eAceType	    GetType() const { return m_eatAceType; }
    const CSid &    GetSid() const { return m_Sid; }
    DWORD	    GetFlags() const { return m_dwAceFlags; }
    DWORD	    GetMask() const { return m_dwMask; }
    const GUID &    GetObjectType() const { return m_guidObject; }
    const GUID &    GetInheritedObjectType() const { return m_guidInheritedObject; }

    BYTE            GetRawAceType() const;
    
    // Is
    BOOL	IsAllowed() const { return (m_eatAceType==AT_AccessAllowed); }
    BOOL	IsDenied() const { return (m_eatAceType==AT_AccessDenied); }
    BOOL	IsAudit() const { return (m_eatAceType==AT_AuditAccess); }
    BOOL	IsInherited() const { return (m_dwAceFlags & INHERITED_ACE)?TRUE:FALSE; }
    BOOL	HasObjectType() const { return (m_guidObject!=gNullGuid); }
    BOOL	HasInheritedObjectType() const { return (m_guidInheritedObject!=gNullGuid); }
    BOOL        IsObjectTypeAce() const { return HasObjectType() || HasInheritedObjectType(); }
    
    // Misc
    DWORD	AddIntoAcl(PACL pAcl) const;
    DWORD	LengthRequired() const;
    void        Clear();

protected:
    void        SetModified();
    void        SetParent(CAcl * pAcl);
    
protected:
    eAceType    m_eatAceType;
    
    CSid        m_Sid;
    DWORD       m_dwAceFlags;
    DWORD       m_dwMask;
    
    GUID        m_guidObject;
    GUID        m_guidInheritedObject;

    CAcl *      m_pParentAcl; // Reference to the owner CAcl

};

#endif