//
// ACL.h
//
// Created by Marcelo Calbucci
//
// April/2000
//

#ifndef __ACL_H__
#define __ACL_H__

#include "ace.h"
#include "sderror.h"
#include <vector>

using namespace std;

class CSD;

class CAcl
{
public:
    friend CSD;
    friend CAce;

    typedef vector<CAce>::const_iterator const_iterator;
    typedef vector<CAce>::iterator iterator;

    const_iterator begin() const { return m_vAces.begin(); }
    const_iterator end() const { return m_vAces.end(); }
    
    iterator begin() { return m_vAces.begin(); }
    iterator end() { return m_vAces.end(); }

    CAcl();
    CAcl(const CAcl & rhs);
    CAcl(const PACL pAcl);
    
    ~CAcl();
    
    CAcl &      operator=(const CAcl & rhs);
    BOOL        operator==(const CAcl & rhs) const;
    BOOL        operator!=(const CAcl & rhs) const;
    
    // Set
    DWORD       SetAcl(const PACL pAcl);
    void        SetAcl(const CAcl & rhs);
    
    // Contain
    BOOL        ContainsAcl(const CAcl & Acl, BOOL bFullMask = FALSE) const;
    BOOL        ContainsAce(const CAce & Ace, BOOL bFullMask = FALSE) const;
    
    // Add
    void	AddAce(const CAce & Ace, int iPos = -1);
    DWORD	AddAce(const LPVOID pAce, int iPos = -1);
    DWORD	AddAccessAllowed(const PSID pSid, DWORD dwMask, DWORD dwAceFlags = 0, 
				 const GUID & guidObjectType = gNullGuid, 
				 const GUID & guidInheritedObjectType = gNullGuid,
				 int iPos = -1);
    DWORD	AddAccessDenied(const PSID pSid, DWORD dwMask, DWORD dwAceFlags = 0, 
				const GUID & guidObjectType = gNullGuid, 
				const GUID &guidInheritedObjectType = gNullGuid,
				int iPos = -1);
    
    void	AddAccessAllowed(const CSid & sid, DWORD dwMask, DWORD dwAceFlags = 0, 
				 const GUID & guidObjectType = gNullGuid, 
			         const GUID &guidInheritedObjectType = gNullGuid,
				 int iPos = -1);
    void	AddAccessDenied(const CSid & sid, DWORD dwMask, DWORD dwAceFlags = 0, 
	                        const GUID & guidObjectType = gNullGuid, 
	                        const GUID &guidInheritedObjectType = gNullGuid,
	                        int iPos = -1);

    DWORD	AddAuditAccess(const PSID pSid, DWORD dwMask, 
                               DWORD dwAceFlags = FAILED_ACCESS_ACE_FLAG, 
			       const GUID & guidObjectType = gNullGuid, 
			       const GUID &guidInheritedObjectType = gNullGuid,
			       int iPos = -1);
    void	AddAuditAccess(const CSid & sid, DWORD dwMask, 
                               DWORD dwAceFlags = FAILED_ACCESS_ACE_FLAG, 
	                       const GUID & guidObjectType = gNullGuid, 
	                       const GUID &guidInheritedObjectType = gNullGuid,
	                       int iPos = -1);
    
    // Get
    const PACL  GetAcl();
    PACL        GetAclCopy(DWORD iExtraSpace = 0);
    CAce &      Ace(int iPos);

    // Is
    BOOL        IsPresent() const { return m_fIsPresent; }
    
    // Delete
    BOOL        RemoveAce(int iPos, CAce & Ace);
    void        DeleteAce(int iPos);
    void        DeleteAce(iterator it);
    static void DeleteAcl(PACL pACl);
    
    // Specials
    void        RemoveAllAces(BOOL bMakeItNotPresent);
    void        CanonicalSort();
    void        RemoveInheritedAces();
    
    // Misc
    DWORD	AceCount() const;
    DWORD	LengthRequired() const;

protected:
    const PACL  GetAcl(DWORD iExtraSpace);

    void        SetModified();
    void        SetParentSD(CSD * pParentSD) { m_pParentSD = pParentSD; }


protected:
    BOOL	    m_fIsPresent;
    vector<CAce>    m_vAces;

    CSD *           m_pParentSD; // Just a reference to the SD that owns this ACL
    
    // Cache
    BOOL            m_fModified;
    PACL            m_pAcl;
};

class CCanonicalSort
{
public:
    int operator()(const CAce & rhs, const CAce & lhs);
private:
    int GetOrder(const CAce & rhs);
};


#endif