//
// SD.h
//
// Created by Marcelo Calbucci
//
// April/2000
//
// Compile with _WIN32_WINNT=0x0500 and _UNICODE defined

#ifndef __SD_H__
#define __SD_H__

#include "windows.h"
#include "sderror.h"
#include "acl.h"

// Request/Set Security Information:
#define SD_SI_ALL_BUT_SACL (OWNER_SECURITY_INFORMATION | \
                            GROUP_SECURITY_INFORMATION | \
                            DACL_SECURITY_INFORMATION)
#define SD_SI_ALL (SD_SI_ALL_BUT_SACL | SACL_SECURITY_INFORMATION)

// This are the only allowed bits that you can set on a SD Control
// (other bits are read-only)
#define ALL_CONTROL_BITS_OF_INTEREST \
        (SE_DACL_AUTO_INHERIT_REQ | SE_DACL_AUTO_INHERITED | SE_DACL_PROTECTED | \
         SE_SACL_AUTO_INHERIT_REQ | SE_SACL_AUTO_INHERITED | SE_SACL_PROTECTED)

// Include all control bits that we use (it does not include ADS_SD_CONTROL_SE_SELF_RELATIVE)
#define ALL_CONTROL_BITS \
        (ALL_CONTROL_BITS_OF_INTEREST | SE_OWNER_DEFAULTED | SE_GROUP_DEFAULTED | \
         SE_DACL_PRESENT | SE_DACL_DEFAULTED | SE_SACL_PRESENT | SE_SACL_DEFAULTED)


class CAcl;

class CSD
{
public:
    friend CAcl;

    CSD();
    CSD(const CSD & rhs);
    CSD(const PSECURITY_DESCRIPTOR pSd);
    CSD(const PSID pOwner, const PSID pGroup,
        const PACL pDacl, const PACL pSacl);
    CSD(const CSid & sidOwner, const CSid & sidGroup,
        const CAcl & Dacl, const CAcl & Sacl);
    
    ~CSD();
    
    CSD &       operator=(const CSD & rhs);
    BOOL        operator==(const CSD & rhs) const;
    BOOL        operator!=(const CSD & rhs) const;
    
    // Set
    DWORD	SetSD(const PSECURITY_DESCRIPTOR pSd);
    void	SetSD(const CSD & rhs);	
    
    DWORD	SetSD(const PSID pOwner, const PSID pGroup,
                      const PACL pDacl, const PACL pSacl);
    void	SetSD(const CSid & sidOwner, const CSid & sidGroup,
                      const CAcl & Dacl, const CAcl & Sacl);

    DWORD	SetSDFromString(const wstring & wsStringSD);

    void        SetControl(SECURITY_DESCRIPTOR_CONTROL Control);
    
    void	SetDacl(const CAcl & Dacl);
    DWORD	SetDacl(const PACL pDacl);
    void	SetSacl(const CAcl & Sacl);
    DWORD	SetSacl(const PACL pSacl);
    
    void	SetOwner(const CSid & sid);
    DWORD	SetOwner(const PSID pSid);
    void	SetGroup(const CSid & sid);
    DWORD	SetGroup(const PSID pSid);

    void	SetOwners(const CSid & sidOwner, const CSid & sidGroup);
    DWORD	SetOwners(const PSID pSidOwner, const PSID pSidGroup);
    
    // Add
    DWORD	AddAce(const CAce & Ace, int iPos = -1);
    DWORD	AddAce(const LPVOID pAce, int iPos = -1);
    DWORD	AddAccessAllowed(const PSID pSid, DWORD dwMask, DWORD dwAceFlags = 0, 
				 const GUID & guidObjectType = gNullGuid, 
                                 const GUID &guidInheritedObjectType = gNullGuid,
                                 int iPos = -1);
    DWORD	AddAccessDenied(const PSID pSid, DWORD dwMask, DWORD dwAceFlags = 0, 
                                const GUID & guidObjectType = gNullGuid, 
                                const GUID &guidInheritedObjectType = gNullGuid,
                                int iPos = -1);
    DWORD	AddAuditAccess(const PSID pSid, DWORD dwMask, DWORD dwAceFlags = 0, 
                               const GUID & guidObjectType = gNullGuid, 
                               const GUID &guidInheritedObjectType = gNullGuid,
                               int iPos = -1);
    
    void	AddAccessAllowed(const CSid & sid, DWORD dwMask, DWORD dwAceFlags = 0, 
				 const GUID & guidObjectType = gNullGuid, 
                                 const GUID &guidInheritedObjectType = gNullGuid,
                                 int iPos = -1);
    void	AddAccessDenied(const CSid & sid, DWORD dwMask, DWORD dwAceFlags = 0, 
                                const GUID & guidObjectType = gNullGuid, 
                                const GUID &guidInheritedObjectType = gNullGuid,
                                int iPos = -1);
    void	AddAuditAccess(const CSid & sid, DWORD dwMask, DWORD dwAceFlags = 0, 
                               const GUID & guidObjectType = gNullGuid, 
                               const GUID &guidInheritedObjectType = gNullGuid,
                               int iPos = -1);
    
    // Get
    SECURITY_DESCRIPTOR_CONTROL GetControl() const { return m_Control; }
    CAcl &          Dacl();
    CAcl &          Sacl();
    const CSid &    GetOwner() const;
    const CSid &    GetGroup() const;
    
    const PSECURITY_DESCRIPTOR GetSD();

    const PSECURITY_DESCRIPTOR GetSDAbsolute();
    const PSECURITY_DESCRIPTOR GetSDRelative();
    // Get a copy that should be deleted with DeleteSD
    PSECURITY_DESCRIPTOR GetSDAbsoluteCopy(int iExtraDaclBytes = 0, int iExtraSaclBytes = 0);
    PSECURITY_DESCRIPTOR GetSDRelativeCopy(int iExtraDaclBytes = 0, int iExtraSaclBytes = 0);
    
    DWORD        GetStringSD(wstring &wsSD, SECURITY_INFORMATION si = SD_SI_ALL);
    
    static void DeleteSD(PSECURITY_DESCRIPTOR pSD);

    // Is
    BOOL        IsDaclProtected() { return m_Control & SE_DACL_PROTECTED?TRUE:FALSE; }
    BOOL        IsSaclProtected() { return m_Control & SE_SACL_PROTECTED?TRUE:FALSE; }
    
    // Specials
    void        RemoveInheritedAces();
    void        CanonicalSort();
    
    // Misc
    DWORD	LengthRequired() const;
    void	Clear();
    
    // Fill this instance with information from...
    DWORD       GetFileSecurity(LPCTSTR szFileName, 
                                SECURITY_INFORMATION si = SD_SI_ALL);
    DWORD       GetKernelObjectSecurity(HANDLE hObject, 
                                SECURITY_INFORMATION si = SD_SI_ALL);
    DWORD       GetUserObjectSecurity(HANDLE hObject, 
                                SECURITY_INFORMATION si = SD_SI_ALL);
    DWORD       GetServiceObjectSecurity(SC_HANDLE hService, 
                                SECURITY_INFORMATION si = SD_SI_ALL);
    DWORD       GetRegKeySecurity(HKEY hKey, 
                                SECURITY_INFORMATION si = SD_SI_ALL);

    // Get 'this' SD and stamp on the object...
    DWORD       SetFileSecurity(LPCTSTR szFileName, 
                                SECURITY_INFORMATION si = SD_SI_ALL);
    DWORD       SetKernelObjectSecurity(HANDLE hObject, 
                                        SECURITY_INFORMATION si = SD_SI_ALL);
    DWORD       SetUserObjectSecurity(HANDLE hObject, 
                                      SECURITY_INFORMATION si = SD_SI_ALL);
    DWORD       SetServiceObjectSecurity(SC_HANDLE hService, 
                                         SECURITY_INFORMATION si = SD_SI_ALL);
    DWORD       SetRegKeySecurity(HKEY hKey, SECURITY_INFORMATION si = SD_SI_ALL);
    
protected:
    const PSECURITY_DESCRIPTOR GetSDAbsolute(int iExtraDaclBytes, int iExtraSaclBytes);
    const PSECURITY_DESCRIPTOR GetSDRelative(int iExtraDaclBytes, int iExtraSaclBytes);

    void	SetModified();
    DWORD       SetSDFromRelative();
    void        CheckControlFlags();
    
protected:
    SECURITY_DESCRIPTOR_CONTROL m_Control; // TODO

    CSid                        m_SidOwner;
    CSid                        m_SidGroup;
    CAcl                        m_Dacl;
    CAcl                        m_Sacl;
    
    // Cached
    BOOL                    m_fModified;		

    PSECURITY_DESCRIPTOR    m_pSDAbsolute;              
    DWORD                   m_dwAbsExtraDaclSize;
    DWORD                   m_dwAbsExtraSaclSize;

    BOOL                    m_fUseLocalFree;            
    PSECURITY_DESCRIPTOR    m_pSDRelative;              
    DWORD                   m_dwRelExtraDaclSize;
    DWORD                   m_dwRelExtraSaclSize;
};


#endif